unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, TntStdCtrls, ExtCtrls,
  CelikApi;

type
  TForm1 = class(TForm)
    btnCitajKarticu: TButton;
    btnSaveMemoToFile: TButton;
    btnLoadFromFile: TButton;
    mmoPodaci: TTntMemo;
    imgPic: TImage;
    procedure btnCitajKarticuClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnSaveMemoToFileClick(Sender: TObject);
    procedure btnLoadFromFileClick(Sender: TObject);
  private
    procedure DisplayData(const ADocData: TNormDocumentData; AFixPerData: TNormFixedPersonalData; AVarPerData: TNormVariablePersonalData);
    procedure DisplayPicture(const AImg: TStream);
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
  Jpeg;

procedure TForm1.btnCitajKarticuClick(Sender: TObject);
var
  DocData: TNormDocumentData;
  FixedData: TNormFixedPersonalData;
  VariableData: TNormVariablePersonalData;
  ms: TMemoryStream;
begin
  ms := TMemoryStream.Create;
  try
    EidBeginRead('', nil);
    NormReadDocumentData(DocData);
    NormReadFixedPersonalData(FixedData);
    NormReadVariablePersonalData(VariableData);
    NormReadPicture(ms);
    EidEndRead;

    DisplayData(DocData, FixedData, VariableData);
    DisplayPicture(ms);
  finally
    ms.Free;
  end;
end;

procedure TForm1.DisplayData(const ADocData: TNormDocumentData; AFixPerData: TNormFixedPersonalData; AVarPerData: TNormVariablePersonalData);
begin
  mmoPodaci.Clear;
  mmoPodaci.Lines.Add('Reg.no: ' + ADocData.DocRegNo);
  mmoPodaci.Lines.Add('Tip dokumenta: ' + ADocData.DocumentType);
  mmoPodaci.Lines.Add('Datum izdavanja: ' + ADocData.IssuingDate);
  mmoPodaci.Lines.Add('Datum isteka: ' + ADocData.ExpiryDate);
  mmoPodaci.Lines.Add('Izdaje: ' + ADocData.IssuingAuthority);

  mmoPodaci.Lines.Add('-----------------------------------------');
  mmoPodaci.Lines.Add('Prezime: ' + AFixPerData.Surname);
  mmoPodaci.Lines.Add('Ime: ' + AFixPerData.GivenName);
  mmoPodaci.Lines.Add('Ime roditelja: ' + AFixPerData.ParentGivenName);
  mmoPodaci.Lines.Add('Pol: ' + AFixPerData.Sex);
  mmoPodaci.Lines.Add('Mesto roenja: ' + AFixPerData.PlaceOfBirth);
  mmoPodaci.Lines.Add('Drava: ' + AFixPerData.StateOfBirth);
  mmoPodaci.Lines.Add('Datum roenja: ' + AFixPerData.DateOfBirth);
  mmoPodaci.Lines.Add('Optina: ' + AFixPerData.CommunityOfBirth);

  mmoPodaci.Lines.Add('-----------------------------------------');
  mmoPodaci.Lines.Add('Prebivalite - oznaka drave: ' + AVarPerData.state);
  mmoPodaci.Lines.Add('Prebivalite - optina: ' + AVarPerData.community);
  mmoPodaci.Lines.Add('Prebivalite - mesto: ' + AVarPerData.place);
  mmoPodaci.Lines.Add('Prebivalite - ulica: ' + AVarPerData.street);
  mmoPodaci.Lines.Add('Prebivalite - kuni broj: ' + AVarPerData.houseNumber);
  mmoPodaci.Lines.Add('Prebivalite - slovo uz broj: ' + AVarPerData.houseLetter);
  mmoPodaci.Lines.Add('Prebivalite - ulaz: ' + AVarPerData.entrance);
  mmoPodaci.Lines.Add('Prebivalite - sprat: ' + AVarPerData.floor);
  mmoPodaci.Lines.Add('Prebivalite - broj stana: ' + AVarPerData.apartmentNumber);
  mmoPodaci.Lines.Add('Prebivalite - promena datuma: ' + AVarPerData.AddressDate);
  mmoPodaci.Lines.Add('Prebivalite - adresna oznaka: ' + AVarPerData.AddressLabel);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  EidStartup(2);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  EidCleanup;
end;

procedure TForm1.btnSaveMemoToFileClick(Sender: TObject);
var
  DocData: TEidDocumentData;
  FixedData: TEidFixedPersonalData;
  VariableData: TEidVariablePersonalData;
  f: TFileStream;
begin
  FillChar(DocData, SizeOf(TEidDocumentData), 0);
  FillChar(FixedData, SizeOf(TEidFixedPersonalData), 0);
  FillChar(VariableData, SizeOf(TEidVariablePersonalData), 0);

  EidBeginRead('', nil);
  EidReadDocumentData(@DocData);
  EidReadFixedPersonalData(@FixedData);
  EidReadVariablePersonalData(@VariableData);
  EidEndRead;

  f := TFileStream.Create('Data.dat', fmCreate);
  try
    f.Write(DocData, SizeOf(TEidDocumentData));
    f.Write(FixedData, SizeOf(TEidFixedPersonalData));
    f.Write(VariableData, SizeOf(TEidVariablePersonalData));
  finally
    f.Free;
  end;
end;

procedure TForm1.btnLoadFromFileClick(Sender: TObject);
var
  DocData: TEidDocumentData;
  FixedData: TEidFixedPersonalData;
  VariableData: TEidVariablePersonalData;
  f: TFileStream;
begin
  f := TFileStream.Create('Data.dat', fmOpenRead);
  try
    f.Read(DocData, SizeOf(TEidDocumentData));
    f.Read(FixedData, SizeOf(TEidFixedPersonalData));
    f.Read(VariableData, SizeOf(TEidVariablePersonalData));
  finally
    f.Free;
  end;
end;

procedure TForm1.DisplayPicture(const AImg: TStream);
var
  img: TJPEGImage;
begin
  if AImg.Size > 0 then
  begin
    img := TJPEGImage.Create;
    try
      AImg.Position := 0;
      img.LoadFromStream(AImg);
      imgPic.Picture.Assign(img);
    finally
      img.Free;
    end;
  end;
end;

end.

